#include<stdio.h>
#include<math.h>

#include<cv.h>
#include<cxcore.h>
#include<highgui.h>


int main(int argc, char **argv) {
    
    printf("hello opencv!\n");
    
    IplImage *first = cvLoadImage("mondrian.bmp", CV_LOAD_IMAGE_GRAYSCALE);
    assert (first != 0);
    IplImage *edge = cvCreateImage(cvGetSize(first), IPL_DEPTH_8U, 1);
    assert (edge != 0);
    cvCanny (first, edge, 10, 100, 3);

    cvNamedWindow("First");
    cvShowImage("First", first);
    cvNamedWindow("Edge");
    cvShowImage("Edge", edge);
    cvWaitKey(0);

    const int nAngles = 360;
    const int nRadii = 360;
    const double pi = 3.14152128;
    IplImage* accumulator = cvCreateImage(cvSize(nAngles,nRadii), IPL_DEPTH_32S, 1);
    memset(accumulator->imageData, 0, accumulator->widthStep*accumulator->height);

    /* scan pixels */
    unsigned char *ptr = (unsigned char *)edge->imageData;
    for (int i = 0; i < edge->height; i++) {
        for (int j = 0; j < edge->width; j++, ptr++) {
            if (*ptr == 255) {
                for (int theta = 0; theta < nAngles; theta ++) {
                    const double th = double(theta)/nAngles * (pi + pi/16) - pi/32;
                    const double x = double(j) / edge->width;
                    const double y = double(i) / edge->height;
                    double r = y * sin(th) + x * cos(th);
                    r /= (2 * sqrt(2.0)); /* -.5 and .5 */
                    r *= nRadii;
                    r += nRadii/2; /* should be >0 now */
                    if (r < 0.0 || r >= nRadii) printf ("r out of range, %f\n", r);
                    else
                        (*((int *)(accumulator->imageData + theta * sizeof(int) + int(r+.5) * accumulator->widthStep))) ++;
                }
            }
        }

        ptr += (edge->widthStep - edge->width);
    }

    IplImage *visual = cvCreateImage(cvGetSize(accumulator), IPL_DEPTH_8U, 1);
    int max = 0;
    for (int i = 0; i < accumulator->height; i++) {
        int *p = (int *)(accumulator->imageData + i * accumulator->widthStep);
        for (int j = 0; j < accumulator->width; j++, p++) {
            if (*p > max) max = *p;
        }
    }

    ptr = (unsigned char *)visual->imageData;
    for (int i = 0; i < accumulator->height; i++) {
        int *p = (int *)(accumulator->imageData + i * accumulator->widthStep);
        for (int j = 0; j < accumulator->width; j++, p++) {
            *ptr++ = (unsigned char)(double(*p) / max * 255.0);
        }
        ptr += (visual->widthStep - visual->width);
    }

    cvNamedWindow("Accumulator");
    cvShowImage("Accumulator", visual);
    cvWaitKey(0);

    const unsigned char threshold = 100;
    ptr = (unsigned char *)visual->imageData;
    for (int i = 0; i < visual->height; i++) {
        for (int j = 0; j < visual->widthStep; j++, ptr++) {
            if (*ptr > threshold)
                printf("found max: r=%d theta=%d\n", i, j);
                /* visualizes lines here */
        }
    }

    /* release memory */
    cvReleaseImage(&first);
    cvDestroyWindow("First");
    cvReleaseImage(&edge);
    cvDestroyWindow("Edge");
    cvReleaseImage(&accumulator);
    cvReleaseImage(&visual);
    cvDestroyWindow("Accumulator");

    return 0;
}
